"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const JsonSource_1 = require("./JsonSource");
const CommandLineSource_1 = require("./CommandLineSource");
const EnvironmentSource_1 = require("./EnvironmentSource");
const YamlSource_1 = require("./YamlSource");
const PropertiesSource_1 = require("./PropertiesSource");
const fs_1 = require("fs");
const path_1 = require("path");
const NullSource_1 = require("./NullSource");
const env = process.env.NODE_ENV || 'development';
const configFileNames = [
    'defaults.env',
    'defaults.json',
    'defaults.yaml',
    `${env}.env`,
    `${env}.json`,
    `${env}.yaml`,
];
class Configuration {
    constructor(options = {}) {
        this.sources = [];
        this.properties = {};
        this.logger = options.logger || console;
        const env = process.env.NODE_ENV || 'development';
        if (options.sources) {
            this.sources = options.sources;
        }
        else {
            try {
                const files = fs_1.readdirSync('config');
                const configFiles = configFileNames
                    .filter(file => files.indexOf(file) >= 0);
                const fileSources = configFiles.map((file) => {
                    const path = path_1.join(process.cwd(), 'config', file);
                    const extIndex = file.lastIndexOf('.');
                    if (extIndex === -1) {
                        return new NullSource_1.NullSource();
                    }
                    switch (file.substr(extIndex).toLowerCase()) {
                        case '.env':
                            return new PropertiesSource_1.PropertiesSource(path, this.logger);
                        case '.json':
                            return new JsonSource_1.JsonSource(path, this.logger);
                        case '.yaml':
                        case '.yml':
                            return new YamlSource_1.YamlSource(path, this.logger);
                        default:
                            return new NullSource_1.NullSource();
                    }
                }).filter(s => !!s);
                this.sources = [
                    ...fileSources,
                    new EnvironmentSource_1.EnvironmentSource(),
                    new CommandLineSource_1.CommandLineSource(),
                ];
            }
            catch (e) {
                this.logger.error(`Failed to load configuration: ${e}`);
                this.sources = [];
            }
        }
    }
    static normalizeKey(key) {
        return key.toLowerCase().replace(/[^a-z0-9]/g, '.');
    }
    init() {
        this.load(...this.sources);
    }
    load(...sources) {
        sources.forEach(s => {
            const props = s.load();
            for (let i = 0; i < props.length; i++) {
                const prop = props[i];
                this.set(prop.key, prop.value);
            }
        });
    }
    get(key) {
        if (key === undefined) {
            return this.properties;
        }
        const normalized = Configuration.normalizeKey(key);
        const parts = normalized.split('.');
        let currentValue = this.properties;
        for (let i = 0; i < parts.length; i++) {
            const part = parts[i];
            if (currentValue instanceof Object && part in currentValue) {
                currentValue = currentValue[part];
            }
            else {
                return undefined;
            }
        }
        return currentValue;
    }
    set(key, value) {
        const parts = Configuration.normalizeKey(key).split('.');
        let currentValue = this.properties;
        for (let i = 0; i < parts.length - 1 && currentValue instanceof Object; i++) {
            const part = parts[i];
            if (!(part in currentValue)) {
                currentValue[part] = {};
            }
            currentValue = currentValue[part];
        }
        if (currentValue instanceof Object) {
            currentValue[parts[parts.length - 1]] = value;
        }
    }
}
exports.Configuration = Configuration;
